/* Gnome Sound System
 * Interface for defining and playing sound UI events
 * such as startup sound, new file, new game etc...
 *
 * Author: Michal Gomulinski <M.Gomulinski@writeme.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * TODO list:
 * 0) Write configuration utility !!!
 * 
 * 1) Make stock sounds definitions come from separate file
 *    (with less prioriy than user's own configuration)
 *
 * 2) Add ability to play *.wav files
 *
 * 3) Add ability to play midi and other long audio clips (mp3?)
 *    (we must have a separate process to play those, as
 *    it shouldn't block the caller for the time of playing)
 *
 * 4) Make SOUND_DIR be a colon separated list of directories
 *    instead of just one directory.
 */

#include "gnome-sound.h"
#include <libgnome/libgnome.h>

/* Real paths (on disk) */
#define DEVAUDIO "/dev/audio"
#define DEFAULT_SOUND_DIR "/usr/local/share/gnome/sounds"

/* Paths in configs */
#define SOUNDCONF "/sound-events/"
#define STOCK "stock"
#define GENERAL "General/"
#define SOUND_DIR_PATH SOUNDCONF GENERAL "Dir=" DEFAULT_SOUND_DIR
#define SOUND_ENAB_PATH SOUNDCONF GENERAL "Enabled=true"
#define DEFAULT_SOUND "=default.au"


static gboolean conf_valid=FALSE;
static gboolean sound_enabled=FALSE;
static GHashTable *sounds=NULL;
static gchar *sound_dir=NULL;

gboolean gnome_sound_app_configured()
{
  gchar *buf;
  gboolean ret;

  buf=g_strconcat(SOUNDCONF, gnome_app_id, NULL);
  ret=gnome_config_has_section(buf);
  g_free(buf);
  return ret;
}

void gnome_sound_app_configure()
{
}

void _gnome_sound_new(gchar *appname, gchar *event, gchar *audiofile)
{
  gchar *path;

  if (appname) 
    path=g_strconcat(SOUNDCONF, appname, "/", event, NULL);
  else
    path=g_strconcat(SOUNDCONF, STOCK, "/", event, NULL);
  gnome_config_set_string(path, audiofile);
  g_free(path);
}

void gnome_sound_set(gchar *appname, gchar *event, gchar *audiofile)
{
  _gnome_sound_new(appname, event, audiofile);
}

gchar *gnome_sound_get(gchar *appname, gchar *event)
{
  gchar *path, *ret;;

  if (appname) 
    path=g_strconcat(SOUNDCONF, appname, "/", event, NULL);
  else
    path=g_strconcat(SOUNDCONF, STOCK, "/", event, NULL);
  ret=gnome_config_get_string(path);
  g_free(path);

  return ret;
}

GList *gnome_sound_list(gchar *appname)
{
  gchar *path, *ev=NULL, *val=NULL;
  GList *ret;
  gpointer iter;

  if (appname)
    path=g_strconcat(SOUNDCONF, appname, NULL);
  else
    path=g_strconcat(SOUNDCONF, STOCK, NULL);
  
  iter=gnome_config_init_iterator(path);
  g_free(path); path=NULL;
  iter=gnome_config_iterator_next(iter, &ev, &val);
  ret=g_list_append(NULL, ev);
  while(iter) {
    g_free(val); val=NULL;
    iter=gnome_config_iterator_next(iter, &ev, &val);
    ret=g_list_append(ret, ev);
  }
  g_free(val); val=NULL;

  return ret;
}

void _gnome_sound_play(gchar *appname, gchar *event)
{
  FILE *audio;
  FILE *clip;
  gchar *path;
  gchar *audiofname, *audiopath;
  gint nbytes;
  gchar buf[1024];
  gboolean new=TRUE;
  gboolean not_configured=FALSE;

  if (!conf_valid) {
    sound_enabled=gnome_config_get_bool_with_default(SOUND_ENAB_PATH, &not_configured);
    sound_dir=gnome_config_get_string(SOUND_DIR_PATH);
    if (sound_enabled) sounds=g_hash_table_new(g_str_hash, g_str_equal);
    if (not_configured) {
      not_configured=FALSE;
      gnome_config_set_bool(SOUND_ENAB_PATH, TRUE);
      gnome_config_set_string(SOUND_DIR_PATH, DEFAULT_SOUND_DIR);
      gnome_config_sync();
    }
    conf_valid=TRUE;
  }

  if (sound_enabled) {
    if (appname)
      path=g_strconcat(SOUNDCONF, appname, "/", event, DEFAULT_SOUND, NULL);
    else
      path=g_strconcat(SOUNDCONF, STOCK, "/", event, DEFAULT_SOUND, NULL);

    g_print("%s == ", path);

    /* First checking in memory and then in config file */
    if ((audiofname=g_hash_table_lookup(sounds, path)) == NULL) {
      audiofname=gnome_config_get_string(path);
      new=TRUE;
    } else new=FALSE;

    if (audiofname) {
      if (audiofname[0]=='/') {  /* Absolute path was given or remembered */
	audiopath=audiofname;
	audiofname=NULL;
      } else { /* Relative path --- prepending sound_dir! */
	audiopath=g_concat_dir_and_file(sound_dir, audiofname);
	g_free(audiofname);
	audiofname=NULL;
      }

      /* Updating cache when we've fetched something new from the file*/
      if (new) g_hash_table_insert(sounds, path, audiofname);

      g_print("%s\n", audiopath);

      audio=fopen(DEVAUDIO, "a");
      if (audio) {
	clip=fopen(audiopath, "r");
	if (clip) {
	  while (!feof(clip)) { /* Copying audio file to /dev/audio */
	    nbytes=fread(buf, 1, 1024, clip);
	    fwrite(buf, 1, nbytes, audio);
	  }
	}
	if (clip) fclose(clip);
      }
      if (audio) fclose(audio);
      g_free(audiopath);
    }
    g_free(path);
  }
}

